#! /bin/sh
#
# DFE-538TX patch release 1.00
# by ITO Takayuki <yuki@fsinet.or.jp>
# http://hp.vector.co.jp/authors/VA001240/beos/index.en.html
#
# This script modifies BeOS R5.0.1 RTL8139 driver
# for use with D-Link DFE-538TX NIC
#
# 1.00	first release
#
# PCI vendor IDs:
# Realtek	10ech
# SMC		1113h
# D-Link	1186h
#
# PCI device IDs:
# RTL8139	8139h
# RTL8139B	8138h
# SMC 1211TX	1211h
# DFE-538TX	1300h

SRCDIR=/system/add-ons/kernel/drivers
DSTDIR=$HOME/config/add-ons/kernel/drivers
SRCFILE=rtl8139
DSTFILE=rtl8139

cd "`dirname $0`"

alert --warning "This script will apply patch to the RTL8139 driver for use with D-Link DFE-538TX." OK Quit > /dev/null
if [ $? -eq 1 ]
then
	alert --stop "Aborted."
	exit 1
fi

if [ "$HOSTTYPE" = ppc -o ! -f ./bsubst ]
then
	if [ -f bsubst.c ]
	then
		cc -o bsubst bsubst.c
	else
		alert --stop "bsubst is missing."
		exit 1
	fi
fi

# check original driver

if [ -f $DSTDIR/bin/$DSTFILE ]
then
	alert --stop "DFE-538TX patch is already applied."
	exit 1
fi
if [ ! -f $SRCDIR/bin/$SRCFILE ]
then
	alert --stop "Original driver $SRCDIR/bin/$SRCFILE not present."
	exit 1
fi

# chipset detection

TMPFILE=/tmp/poke.log

poke > $TMPFILE <<END
pci
quit
END

ID=`grep 'vendor 1186' $TMPFILE | head -1 | cut -d ' ' -f 10`
rm $TMPFILE
if [ "x$ID" = x ]
then
	alert --stop "No D-Link product detected."
	exit 1
fi

case $ID in
1300)	CHIP=DFE-538TX;;
*)	CHIP=unknown;;
esac
if [ $CHIP = unknown ]
then
	alert --info "Unknown D-Link product detected. Device ID is $ID."
else
	alert --info "D-Link Ethernet card $CHIP detected. Device ID is $ID."
fi

# apply patch to driver

if [ -f $SRCDIR/bin/$SRCFILE.gz ]
then
	alert --stop "Patch is already applied."
	exit 1
fi

alert --info "Processing $SRCDIR/bin/$SRCFILE."
< $SRCDIR/bin/$SRCFILE \
./bsubst "13 11 11 12" "86 11 00 13" \
> $DSTDIR/bin/$DSTFILE

cmp -s $SRCDIR/bin/$SRCFILE $DSTDIR/bin/$DSTFILE
if [ $? -eq 0 ]
then
	alert --stop "Patch is not applicable. Nothing altered."
	rm $DSTDIR/bin/$DSTFILE
	exit 1
fi

gzip $SRCDIR/bin/$SRCFILE
[ -e $DSTDIR/dev/net/$DSTFILE ] || ln -s ../../bin/$DSTFILE $DSTDIR/dev/net/$DSTFILE

alert --info "Done."
exit 0
